@@ -43,22 +43,22 @@ DocSets::~DocSets()
 void DocSets::initialize()
 {
   // -- get config options
-  QCString projectName = Config_getString("PROJECT_NAME");
+  QCString projectName = Config_getString(PROJECT_NAME);
   if (projectName.isEmpty()) projectName="root";
-  QCString bundleId = Config_getString("DOCSET_BUNDLE_ID");
+  QCString bundleId = Config_getString(DOCSET_BUNDLE_ID);
   if (bundleId.isEmpty()) bundleId="org.doxygen.Project";
-  QCString feedName = Config_getString("DOCSET_FEEDNAME");
+  QCString feedName = Config_getString(DOCSET_FEEDNAME);
   if (feedName.isEmpty()) feedName="FeedName";
-  QCString publisherId = Config_getString("DOCSET_PUBLISHER_ID");
+  QCString publisherId = Config_getString(DOCSET_PUBLISHER_ID);
   if (publisherId.isEmpty()) publisherId="PublisherId";
-  QCString publisherName = Config_getString("DOCSET_PUBLISHER_NAME");
+  QCString publisherName = Config_getString(DOCSET_PUBLISHER_NAME);
   if (publisherName.isEmpty()) publisherName="PublisherName";
-  QCString projectNumber = Config_getString("PROJECT_NUMBER");
+  QCString projectNumber = Config_getString(PROJECT_NUMBER);
   if (projectNumber.isEmpty()) projectNumber="ProjectNumber";
 
   // -- write Makefile
   {
-  QCString mfName = Config_getString("HTML_OUTPUT") + "/Makefile";
+  QCString mfName = Config_getString(HTML_OUTPUT) + "/Makefile";
   QFile makefile(mfName);
   if (!makefile.open(IO_WriteOnly))
   {
@@ -109,7 +109,7 @@ void DocSets::initialize()
 
   // -- write Info.plist
   {
-  QCString plName = Config_getString("HTML_OUTPUT") + "/Info.plist";
+  QCString plName = Config_getString(HTML_OUTPUT) + "/Info.plist";
   QFile plist(plName);
   if (!plist.open(IO_WriteOnly))
   {
@@ -145,14 +145,14 @@ void DocSets::initialize()
   }
 
   // -- start Nodes.xml
-  QCString notes = Config_getString("HTML_OUTPUT") + "/Nodes.xml";
+  QCString notes = Config_getString(HTML_OUTPUT) + "/Nodes.xml";
   m_nf = new QFile(notes);
   if (!m_nf->open(IO_WriteOnly))
   {
     err("Could not open file %s for writing\n",notes.data());
     exit(1);
   }
-  //QCString indexName=Config_getBool("GENERATE_TREEVIEW")?"main":"index";
+  //QCString indexName=Config_getBool(GENERATE_TREEVIEW)?"main":"index";
   QCString indexName="index";
   m_nts.setDevice(m_nf);
   m_nts << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
@@ -166,7 +166,7 @@ void DocSets::initialize()
   m_firstNode.resize(m_dc);
   m_firstNode.at(0)=TRUE;
 
-  QCString tokens = Config_getString("HTML_OUTPUT") + "/Tokens.xml";
+  QCString tokens = Config_getString(HTML_OUTPUT) + "/Tokens.xml";
   m_tf = new QFile(tokens);
   if (!m_tf->open(IO_WriteOnly))
   {
@@ -236,7 +236,7 @@ void DocSets::addContentsItem(bool isDir,
                               const char *anchor,
                               bool /* separateIndex */,
                               bool /* addToNavIndex */,
-                              Definition * /*def*/)
+                              const Definition * /*def*/)
 {
   (void)isDir;
   //printf("DocSets::addContentsItem(%s) m_dc=%d\n",name,m_dc);
@@ -274,14 +274,14 @@ void DocSets::addContentsItem(bool isDir,
   }
 }
 
-void DocSets::addIndexItem(Definition *context,MemberDef *md,
+void DocSets::addIndexItem(const Definition *context,const MemberDef *md,
                            const char *,const char *)
 {
   if (md==0 && context==0) return;
 
-  FileDef *fd      = 0;
-  ClassDef *cd     = 0;
-  NamespaceDef *nd = 0;
+  const FileDef *fd      = 0;
+  const ClassDef *cd     = 0;
+  const NamespaceDef *nd = 0;
 
   if (md)
   {
@@ -331,8 +331,10 @@ void DocSets::addIndexItem(Definition *context,MemberDef *md,
     case SrcLangExt_Fortran: lang="fortran"; break;    // Fortran
     case SrcLangExt_VHDL:    lang="vhdl"; break;       // VHDL
     case SrcLangExt_XML:     lang="xml"; break;        // DBUS XML
+    case SrcLangExt_SQL:     lang="sql"; break;        // Sql
     case SrcLangExt_Tcl:     lang="tcl"; break;        // Tcl
     case SrcLangExt_Markdown:lang="markdown"; break;   // Markdown
+    case SrcLangExt_Slice:   lang="slice"; break;      // Slice
     case SrcLangExt_Unknown: lang="unknown"; break;    // should not happen!
   }
 
@@ -400,6 +402,10 @@ void DocSets::addIndexItem(Definition *context,MemberDef *md,
         type="ifc"; break;
       case MemberType_Service:
         type="svc"; break;
+      case MemberType_Sequence:
+        type="sequence"; break;
+      case MemberType_Dictionary:
+        type="dictionary"; break;
     }
     cd = md->getClassDef();
     nd = md->getNamespaceDef();
@@ -411,7 +417,7 @@ void DocSets::addIndexItem(Definition *context,MemberDef *md,
     {
       scope = nd->name();
     }
-    MemberDef *declMd = md->memberDeclaration();
+    const MemberDef *declMd = md->memberDeclaration();
     if (declMd==0) declMd = md;
     {
       fd = md->getFileDef();
@@ -426,15 +432,15 @@ void DocSets::addIndexItem(Definition *context,MemberDef *md,
   {
     if (fd==0 && context->definitionType()==Definition::TypeFile)
     {
-      fd = (FileDef*)context;
+      fd = dynamic_cast<const FileDef*>(context);
     }
     if (cd==0 && context->definitionType()==Definition::TypeClass)
     {
-      cd = (ClassDef*)context;
+      cd = dynamic_cast<const ClassDef*>(context);
     }
     if (nd==0 && context->definitionType()==Definition::TypeNamespace)
     {
-      nd = (NamespaceDef*)context;
+      nd = dynamic_cast<const NamespaceDef*>(context);
     }
     if (fd)
     {
@@ -468,10 +474,6 @@ void DocSets::addIndexItem(Definition *context,MemberDef *md,
       if (ii)
       {
         decl=ii->includeName;
-        if (decl.isEmpty())
-        {
-          decl=ii->local;
-        }
       }
     }
     else if (nd)
