@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -15,21 +15,24 @@
  *
  */
 
-#ifndef CODE_H
-#define CODE_H
+#ifndef FORTRANCODE_H
+#define FORTRANCODE_H
 
-#include "qtbc.h"
-#include <stdio.h>
+#include "types.h"
 
 class CodeOutputInterface;
 class FileDef;
 class MemberDef;
+class QCString;
+class Definition;
 
 void parseFortranCode(CodeOutputInterface &,const char *,const QCString &, 
             bool ,const char *,FileDef *fd,
             int startLine,int endLine,bool inlineFragment,
-            MemberDef *memberDef,bool showLineNumbers,Definition *searchCtx);
+            const MemberDef *memberDef,bool showLineNumbers,const Definition *searchCtx,
+            bool collectRefs, FortranFormat format);
 void resetFortranCodeParserState();
 void codeFreeScanner();
 
+const int fixedCommentAfter = 72;
 #endif
