@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: latexgen.h,v 1.50 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -22,6 +22,51 @@
 
 class QFile;
 
+static const char *latexStyleExtension = ".sty";
+
+class LatexCodeGenerator : public CodeOutputInterface
+{
+  public:
+    LatexCodeGenerator(FTextStream &t,const QCString &relPath,const QCString &sourceFile);
+    LatexCodeGenerator();
+    void setTextStream(FTextStream &t);
+    void setRelativePath(const QCString &path);
+    void setSourceFileName(const QCString &sourceFileName);
+    void codify(const char *text);
+    void writeCodeLink(const char *ref,const char *file,
+                       const char *anchor,const char *name,
+                       const char *tooltip);
+    void writeTooltip(const char *,
+                      const DocLinkInfo &,
+                      const char *,
+                      const char *,
+                      const SourceLinkInfo &,
+                      const SourceLinkInfo &
+                     ) {}
+    void writeLineNumber(const char *,const char *,const char *,int);
+    void startCodeLine(bool);
+    void endCodeLine();
+    void startFontClass(const char *);
+    void endFontClass();
+    void writeCodeAnchor(const char *) {}
+    void setCurrentDoc(const Definition *,const char *,bool) {}
+    void addWord(const char *,bool) {}
+    static void setDoxyCodeOpen(bool val);
+
+  private:
+    void _writeCodeLink(const char *className,
+                        const char *ref,const char *file,
+                        const char *anchor,const char *name,
+                        const char *tooltip);
+    void docify(const char *str);
+    bool m_streamSet;
+    FTextStream m_t;
+    QCString m_relPath;
+    QCString m_sourceFileName;
+    int m_col;
+    bool m_prettyCode;
+};
+
 /** Generator for LaTeX output. */
 class LatexGenerator : public OutputGenerator
 {
@@ -45,7 +90,33 @@ class LatexGenerator : public OutputGenerator
     bool isEnabled(OutputType o) { return (o==Latex && active); } 
     OutputGenerator *get(OutputType o) { return (o==Latex) ? this : 0; }
 
-    void writeDoc(DocNode *,Definition *ctx,MemberDef *);
+    // --- CodeOutputInterface
+    void codify(const char *text)
+    { m_codeGen.codify(text); }
+    void writeCodeLink(const char *ref, const char *file,
+                       const char *anchor,const char *name,
+                       const char *tooltip)
+    { m_codeGen.writeCodeLink(ref,file,anchor,name,tooltip); }
+    void writeLineNumber(const char *ref,const char *file,const char *anchor,int lineNumber)
+    { m_codeGen.writeLineNumber(ref,file,anchor,lineNumber); }
+    void writeTooltip(const char *id, const DocLinkInfo &docInfo, const char *decl,
+                      const char *desc, const SourceLinkInfo &defInfo, const SourceLinkInfo &declInfo
+                     )
+    { m_codeGen.writeTooltip(id,docInfo,decl,desc,defInfo,declInfo); }
+    void startCodeLine(bool hasLineNumbers)
+    { m_codeGen.startCodeLine(hasLineNumbers); }
+    void endCodeLine()
+    { m_codeGen.endCodeLine(); }
+    void startFontClass(const char *s)
+    { m_codeGen.startFontClass(s); }
+    void endFontClass()
+    { m_codeGen.endFontClass(); }
+    void writeCodeAnchor(const char *anchor)
+    { m_codeGen.writeCodeAnchor(anchor); }
+    // ---------------------------
+
+
+    void writeDoc(DocNode *,const Definition *ctx,const MemberDef *);
 
     void startFile(const char *name,const char *manName,const char *title);
     void writeSearchInfo() {}
@@ -65,7 +136,7 @@ class LatexGenerator : public OutputGenerator
     void endTitle()   { t << "}"; }
 
     void newParagraph();
-    void startParagraph();
+    void startParagraph(const char *classDef);
     void endParagraph();
     void writeString(const char *text);
     void startIndexListItem() {}
@@ -81,12 +152,9 @@ class LatexGenerator : public OutputGenerator
     void startIndexItem(const char *ref,const char *file);
     void endIndexItem(const char *ref,const char *file);
     void docify(const char *text);
-    void codify(const char *text);
     void writeObjectLink(const char *ref,const char *file,
                          const char *anchor,const char *name);
-    void writeCodeLink(const char *ref, const char *file,
-                       const char *anchor,const char *name,
-                       const char *tooltip);
+
     void startTextLink(const char *,const char *);
     void endTextLink();
     void startHtmlLink(const char *url);
@@ -102,7 +170,7 @@ class LatexGenerator : public OutputGenerator
     void endMemberSections() {} 
     void startHeaderSection() {}
     void endHeaderSection() {}
-    void startMemberHeader(const char *);
+    void startMemberHeader(const char *,int);
     void endMemberHeader();
     void startMemberSubtitle() {}
     void endMemberSubtitle() {}
@@ -127,14 +195,12 @@ class LatexGenerator : public OutputGenerator
     void endMemberGroup(bool);
     
     void insertMemberAlign(bool) {}
+    void insertMemberAlignLeft(int,bool){}
 
     void writeRuler() { t << endl << endl; }
     void writeAnchor(const char *fileName,const char *name);
     void startCodeFragment();
     void endCodeFragment();
-    void writeLineNumber(const char *,const char *,const char *,int l);
-    void startCodeLine(bool hasLineNumbers);
-    void endCodeLine();
     void startEmphasis() { t << "{\\em ";  }
     void endEmphasis()   { t << "}"; }
     void startBold()     { t << "{\\bfseries "; }
@@ -144,12 +210,10 @@ class LatexGenerator : public OutputGenerator
     void startDescItem();
     void endDescItem();
     void lineBreak(const char *style=0);
-    void startMemberDoc(const char *,const char *,const char *,const char *,bool);
+    void startMemberDoc(const char *,const char *,const char *,const char *,int,int,bool);
     void endMemberDoc(bool);
     void startDoxyAnchor(const char *,const char *,const char *,const char *,const char *);
     void endDoxyAnchor(const char *,const char *);
-    void startCodeAnchor(const char *);
-    void endCodeAnchor();
     void writeChar(char c);
     void writeLatexSpacing() { t << "\\hspace{0.3cm}"; }
     void writeStartAnnoItem(const char *type,const char *file, 
@@ -163,7 +227,7 @@ class LatexGenerator : public OutputGenerator
     void endCenter()        { t << "\\end{center}" << endl; }
     void startSmall()       { t << "\\footnotesize "; }
     void endSmall()         { t << "\\normalsize "; }
-    void startMemberDescription(const char *,const char *);
+    void startMemberDescription(const char *,const char *,bool);
     void endMemberDescription();
     void startMemberDeclaration() {} 
     void endMemberDeclaration(const char *,const char *) {}
@@ -171,8 +235,8 @@ class LatexGenerator : public OutputGenerator
                       const char *,const char *,const char *) {}
     void startDescList(SectionTypes)     { t << "\\begin{Desc}\n\\item["; }
     void endDescList()       { t << "\\end{Desc}" << endl; }
-    void startSimpleSect(SectionTypes,const char *,const char *,const char *);
-    void endSimpleSect();
+    void startExamples();
+    void endExamples();
     void startParamList(ParamListTypes,const char *title);
     void endParamList();
     void startDescForItem()     { t << "\\par" << endl; }
@@ -197,36 +261,34 @@ class LatexGenerator : public OutputGenerator
     void startContents() {}
     void endContents() {}
     void writeNonBreakableSpace(int);
-    
-    void startDescTable()
-    { t << "\\begin{description}" << endl; }
-    void endDescTable()
-    { t << "\\end{description}" << endl; }
-    void startDescTableTitle()
-    { t << "\\item[{\\em " << endl; }
-    void endDescTableTitle()
-    { t << "}]"; }
-    void startDescTableData() {}
-    void endDescTableData() {}
-    void lastIndexPage() {}
+
+    void startDescTable(const char *title);
+    void endDescTable();
+    void startDescTableRow();
+    void endDescTableRow();
+    void startDescTableTitle();
+    void endDescTableTitle();
+    void startDescTableData();
+    void endDescTableData();
+    void lastIndexPage();
 
     void startDotGraph();
-    void endDotGraph(const DotClassGraph &);
+    void endDotGraph(DotClassGraph &);
     void startInclDepGraph();
-    void endInclDepGraph(const DotInclDepGraph &);
+    void endInclDepGraph(DotInclDepGraph &);
     void startCallGraph();
     void startGroupCollaboration();
-    void endGroupCollaboration(const DotGroupCollaboration &g);
-    void endCallGraph(const DotCallGraph &);
+    void endGroupCollaboration(DotGroupCollaboration &g);
+    void endCallGraph(DotCallGraph &);
     void startDirDepGraph();
-    void endDirDepGraph(const DotDirDeps &g);
-    void writeGraphicalHierarchy(const DotGfxHierarchyTable &) {}
+    void endDirDepGraph(DotDirDeps &g);
+    void writeGraphicalHierarchy(DotGfxHierarchyTable &) {}
 
     void startTextBlock(bool) {}
     void endTextBlock(bool) {}
 
     void startMemberDocPrefixItem() {}
-    void endMemberDocPrefixItem() {}
+    void endMemberDocPrefixItem() { t << "\\\\" << endl; }
     void startMemberDocName(bool) {}
     void endMemberDocName() {}
     void startParameterType(bool,const char *);
@@ -235,6 +297,7 @@ class LatexGenerator : public OutputGenerator
     void endParameterName(bool,bool,bool);
     void startParameterList(bool);
     void endParameterList();
+    void exceptionEntry(const char*,bool);
 
     void startConstraintList(const char *);
     void startConstraintParam();
@@ -245,8 +308,8 @@ class LatexGenerator : public OutputGenerator
     void endConstraintDocs();
     void endConstraintList();
 
-    void startMemberDocSimple();
-    void endMemberDocSimple();
+    void startMemberDocSimple(bool);
+    void endMemberDocSimple(bool);
     void startInlineMemberType();
     void endInlineMemberType();
     void startInlineMemberName();
@@ -258,26 +321,21 @@ class LatexGenerator : public OutputGenerator
     void writeLabel(const char *l,bool isLast);
     void endLabels();
 
-    void startFontClass(const char *); // {}
-    void endFontClass(); // {}
+    void setCurrentDoc(const Definition *,const char *,bool) {}
+    void addWord(const char *,bool) {}
 
-    void writeCodeAnchor(const char *) {}
-    void linkableSymbol(int,const char *,Definition *,Definition *) {}
 
   private:
     LatexGenerator(const LatexGenerator &);
     LatexGenerator &operator=(const LatexGenerator &);
-    void escapeLabelName(const char *s);
-    void escapeMakeIndexChars(const char *s);
-    int col;
     bool insideTabbing;
     bool firstDescItem;
     bool disableLinks;
     QCString relPath;
-    QCString sourceFileName;
     int m_indent;
     bool templateMemberItem;
     bool m_prettyCode;
+    LatexCodeGenerator m_codeGen;
 };
 
 #endif
