@@ -388,8 +388,6 @@ public:
   void visitPost(DocXRefItem *);
   void visitPre(DocInternalRef *);
   void visitPost(DocInternalRef *);
-  void visitPre(DocCopy *);
-  void visitPost(DocCopy *);
   void visitPre(DocText *);
   void visitPost(DocText *);
   void visitPre(DocHtmlBlockQuote *);
@@ -642,7 +640,9 @@ void PerlModDocVisitor::visit(DocStyleChange *s)
   {
     case DocStyleChange::Bold:          style = "bold"; break;
     case DocStyleChange::Strike:        style = "strike"; break;
+    case DocStyleChange::Del:           style = "del"; break;
     case DocStyleChange::Underline:     style = "underline"; break;
+    case DocStyleChange::Ins:           style = "ins"; break;
     case DocStyleChange::Italic:        style = "italic"; break;
     case DocStyleChange::Code:          style = "code"; break;
     case DocStyleChange::Subscript:     style = "subscript"; break;
@@ -729,6 +729,7 @@ void PerlModDocVisitor::visit(DocInclude *inc)
 #endif
     return;
   case DocInclude::DontInclude:	return;
+  case DocInclude::DontIncWithLines: return;
   case DocInclude::HtmlInclude:	type = "htmlonly"; break;
   case DocInclude::LatexInclude: type = "latexonly"; break;
   case DocInclude::VerbInclude:	type = "preformatted"; break;
@@ -748,7 +749,7 @@ void PerlModDocVisitor::visit(DocInclude *inc)
 void PerlModDocVisitor::visit(DocIncOperator *)
 {
 #if 0
-  //printf("DocIncOperator: type=%d first=%d, last=%d text=`%s'\n",
+  //printf("DocIncOperator: type=%d first=%d, last=%d text='%s'\n",
   //    op->type(),op->isFirst(),op->isLast(),op->text().data());
   if (op->isFirst())
   {
@@ -1384,14 +1385,6 @@ void PerlModDocVisitor::visitPost(DocInternalRef *)
   closeItem();
 }
 
-void PerlModDocVisitor::visitPre(DocCopy *)
-{
-}
-
-void PerlModDocVisitor::visitPost(DocCopy *)
-{
-}
-
 void PerlModDocVisitor::visitPre(DocText *)
 {
 }
@@ -1461,7 +1454,7 @@ static void addMemberTemplateLists(MemberDef *md,PerlModOutput &output)
 }
 #endif
 
-static void addTemplateList(ClassDef *cd,PerlModOutput &output)
+static void addTemplateList(const ClassDef *cd,PerlModOutput &output)
 {
   addTemplateArgumentList(cd->templateArguments(),output,cd->name());
 }
@@ -1470,8 +1463,8 @@ static void addPerlModDocBlock(PerlModOutput &output,
 			    const char *name,
 			    const QCString &fileName,
 			    int lineNr,
-			    Definition *scope,
-			    MemberDef *md,
+			    const Definition *scope,
+			    const MemberDef *md,
 			    const QCString &text)
 {
   QCString stext = text.stripWhiteSpace();
@@ -1542,14 +1535,14 @@ public:
 
   inline PerlModGenerator(bool pretty) : m_output(pretty) { }
 
-  void generatePerlModForMember(MemberDef *md, Definition *);
-  void generatePerlModSection(Definition *d, MemberList *ml,
+  void generatePerlModForMember(const MemberDef *md, const Definition *);
+  void generatePerlModSection(const Definition *d, MemberList *ml,
 			      const char *name, const char *header=0);
-  void addListOfAllMembers(ClassDef *cd);
-  void generatePerlModForClass(ClassDef *cd);
-  void generatePerlModForNamespace(NamespaceDef *nd);
-  void generatePerlModForFile(FileDef *fd);
-  void generatePerlModForGroup(GroupDef *gd);
+  void addListOfAllMembers(const ClassDef *cd);
+  void generatePerlModForClass(const ClassDef *cd);
+  void generatePerlModForNamespace(const NamespaceDef *nd);
+  void generatePerlModForFile(const FileDef *fd);
+  void generatePerlModForGroup(const GroupDef *gd);
   void generatePerlModForPage(PageDef *pi);
   
   bool createOutputFile(QFile &f, const char *s);
@@ -1566,7 +1559,7 @@ public:
   void generate();
 };
 
-void PerlModGenerator::generatePerlModForMember(MemberDef *md,Definition *)
+void PerlModGenerator::generatePerlModForMember(const MemberDef *md,const Definition *)
 {
   // + declaration/definition arg lists
   // + reimplements
@@ -1616,23 +1609,23 @@ void PerlModGenerator::generatePerlModForMember(MemberDef *md,Definition *)
       md->memberType()!=MemberType_Enumeration)
     m_output.addFieldQuotedString("type", md->typeString());
   
-  ArgumentList *al = md->argumentList();
+  const ArgumentList *al = md->argumentList();
   if (isFunc) //function
   {
     m_output.addFieldBoolean("const", al!=0 && al->constSpecifier)
       .addFieldBoolean("volatile", al!=0 && al->volatileSpecifier);
 
     m_output.openList("parameters");
-    ArgumentList *declAl = md->declArgumentList();
-    ArgumentList *defAl  = md->argumentList();
+    const ArgumentList *declAl = md->declArgumentList();
+    const ArgumentList *defAl  = md->argumentList();
     if (declAl && defAl && declAl->count()>0)
     {
       ArgumentListIterator declAli(*declAl);
       ArgumentListIterator defAli(*defAl);
-      Argument *a;
+      const Argument *a;
       for (declAli.toFirst();(a=declAli.current());++declAli)
       {
-	Argument *defArg = defAli.current();
+	const Argument *defArg = defAli.current();
 	m_output.openHash();
 
 	if (!a->name.isEmpty())
@@ -1664,7 +1657,7 @@ void PerlModGenerator::generatePerlModForMember(MemberDef *md,Definition *)
   {
     m_output.openList("parameters");
     ArgumentListIterator ali(*al);
-    Argument *a;
+    const Argument *a;
     for (ali.toFirst();(a=ali.current());++ali)
     {
       m_output.openHash()
@@ -1686,12 +1679,12 @@ void PerlModGenerator::generatePerlModForMember(MemberDef *md,Definition *)
   
   if (md->memberType()==MemberType_Enumeration) // enum
   {
-    MemberList *enumFields = md->enumFieldList();
+    const MemberList *enumFields = md->enumFieldList();
     if (enumFields)
     {
       m_output.openList("values");
       MemberListIterator emli(*enumFields);
-      MemberDef *emd;
+      const MemberDef *emd;
       for (emli.toFirst();(emd=emli.current());++emli)
       {
 	m_output.openHash()
@@ -1710,7 +1703,7 @@ void PerlModGenerator::generatePerlModForMember(MemberDef *md,Definition *)
     }
   }
 
-  MemberDef *rmd = md->reimplements();
+  const MemberDef *rmd = md->reimplements();
   if (rmd)
     m_output.openHash("reimplements")
       .addFieldQuotedString("name", rmd->name())
@@ -1731,7 +1724,7 @@ void PerlModGenerator::generatePerlModForMember(MemberDef *md,Definition *)
   m_output.closeHash();
 }
 
-void PerlModGenerator::generatePerlModSection(Definition *d,
+void PerlModGenerator::generatePerlModSection(const Definition *d,
 					      MemberList *ml,const char *name,const char *header)
 {
   if (ml==0) return; // empty list
@@ -1743,7 +1736,7 @@ void PerlModGenerator::generatePerlModSection(Definition *d,
   
   m_output.openList("members");
   MemberListIterator mli(*ml);
-  MemberDef *md;
+  const MemberDef *md;
   for (mli.toFirst();(md=mli.current());++mli)
   {
     generatePerlModForMember(md,d);
@@ -1752,7 +1745,7 @@ void PerlModGenerator::generatePerlModSection(Definition *d,
     .closeHash();
 }
 
-void PerlModGenerator::addListOfAllMembers(ClassDef *cd)
+void PerlModGenerator::addListOfAllMembers(const ClassDef *cd)
 {
   m_output.openList("all_members");
   if (cd->memberNameInfoSDict())
@@ -1765,9 +1758,9 @@ void PerlModGenerator::addListOfAllMembers(ClassDef *cd)
       MemberInfo *mi;
       for (mii.toFirst();(mi=mii.current());++mii)
       {
-        MemberDef *md=mi->memberDef;
-        ClassDef  *cd=md->getClassDef();
-        Definition *d=md->getGroupDef();
+        const MemberDef *md=mi->memberDef;
+        const ClassDef  *cd=md->getClassDef();
+        const Definition *d=md->getGroupDef();
         if (d==0) d = cd;
 
         m_output.openHash()
@@ -1786,7 +1779,7 @@ void PerlModGenerator::addListOfAllMembers(ClassDef *cd)
   m_output.closeList();
 }
 
-void PerlModGenerator::generatePerlModForClass(ClassDef *cd)
+void PerlModGenerator::generatePerlModForClass(const ClassDef *cd)
 {
   // + brief description
   // + detailed description
@@ -1844,7 +1837,7 @@ void PerlModGenerator::generatePerlModForClass(ClassDef *cd)
   {
     m_output.openList("inner");
     ClassSDict::Iterator cli(*cl);
-    ClassDef *cd;
+    const ClassDef *cd;
     for (cli.toFirst();(cd=cli.current());++cli)
       m_output.openHash()
 	.addFieldQuotedString("name", cd->name())
@@ -1936,7 +1929,7 @@ void PerlModGenerator::generatePerlModForClass(ClassDef *cd)
   m_output.closeHash();
 }
 
-void PerlModGenerator::generatePerlModForNamespace(NamespaceDef *nd)
+void PerlModGenerator::generatePerlModForNamespace(const NamespaceDef *nd)
 {
   // + contained class definitions
   // + contained namespace definitions
@@ -1957,7 +1950,7 @@ void PerlModGenerator::generatePerlModForNamespace(NamespaceDef *nd)
   {
     m_output.openList("classes");
     ClassSDict::Iterator cli(*cl);
-    ClassDef *cd;
+    const ClassDef *cd;
     for (cli.toFirst();(cd=cli.current());++cli)
       m_output.openHash()
 	.addFieldQuotedString("name", cd->name())
@@ -1965,12 +1958,12 @@ void PerlModGenerator::generatePerlModForNamespace(NamespaceDef *nd)
     m_output.closeList();
   }
 
-  NamespaceSDict *nl = nd->getNamespaceSDict();
+  const NamespaceSDict *nl = nd->getNamespaceSDict();
   if (nl)
   {
     m_output.openList("namespaces");
     NamespaceSDict::Iterator nli(*nl);
-    NamespaceDef *nd;
+    const NamespaceDef *nd;
     for (nli.toFirst();(nd=nli.current());++nli)
       m_output.openHash()
 	.addFieldQuotedString("name", nd->name())
@@ -1981,7 +1974,7 @@ void PerlModGenerator::generatePerlModForNamespace(NamespaceDef *nd)
   if (nd->getMemberGroupSDict())
   {
     MemberGroupSDict::Iterator mgli(*nd->getMemberGroupSDict());
-    MemberGroup *mg;
+    const MemberGroup *mg;
     for (;(mg=mgli.current());++mgli)
       generatePerlModSection(nd,mg->members(),"user-defined",mg->header());
   }
@@ -1999,7 +1992,7 @@ void PerlModGenerator::generatePerlModForNamespace(NamespaceDef *nd)
   m_output.closeHash();
 }
 
-void PerlModGenerator::generatePerlModForFile(FileDef *fd)
+void PerlModGenerator::generatePerlModForFile(const FileDef *fd)
 {
   // + includes files
   // + includedby files
@@ -2068,7 +2061,7 @@ void PerlModGenerator::generatePerlModForFile(FileDef *fd)
   m_output.closeHash();
 }
 
-void PerlModGenerator::generatePerlModForGroup(GroupDef *gd)
+void PerlModGenerator::generatePerlModForGroup(const GroupDef *gd)
 {
   // + members
   // + member groups
@@ -2093,7 +2086,7 @@ void PerlModGenerator::generatePerlModForGroup(GroupDef *gd)
   {
     m_output.openList("files");
     QListIterator<FileDef> fli(*fl);
-    FileDef *fd;
+    const FileDef *fd;
     for (fli.toFirst();(fd=fli.current());++fli)
       m_output.openHash()
 	.addFieldQuotedString("name", fd->name())
@@ -2106,7 +2099,7 @@ void PerlModGenerator::generatePerlModForGroup(GroupDef *gd)
   {
     m_output.openList("classes");
     ClassSDict::Iterator cli(*cl);
-    ClassDef *cd;
+    const ClassDef *cd;
     for (cli.toFirst();(cd=cli.current());++cli)
       m_output.openHash()
 	.addFieldQuotedString("name", cd->name())
@@ -2119,7 +2112,7 @@ void PerlModGenerator::generatePerlModForGroup(GroupDef *gd)
   {
     m_output.openList("namespaces");
     NamespaceSDict::Iterator nli(*nl);
-    NamespaceDef *nd;
+    const NamespaceDef *nd;
     for (nli.toFirst();(nd=nli.current());++nli)
       m_output.openHash()
 	.addFieldQuotedString("name", nd->name())
@@ -2145,7 +2138,7 @@ void PerlModGenerator::generatePerlModForGroup(GroupDef *gd)
   {
     m_output.openList("groups");
     GroupListIterator gli(*gl);
-    GroupDef *sgd;
+    const GroupDef *sgd;
     for (gli.toFirst();(sgd=gli.current());++gli)
       m_output.openHash()
 	.addFieldQuotedString("title", sgd->groupTitle())
@@ -2206,14 +2199,14 @@ bool PerlModGenerator::generatePerlModOutput()
   
   m_output.openList("classes");
   ClassSDict::Iterator cli(*Doxygen::classSDict);
-  ClassDef *cd;
+  const ClassDef *cd;
   for (cli.toFirst();(cd=cli.current());++cli)
     generatePerlModForClass(cd);
   m_output.closeList();
 
   m_output.openList("namespaces");
   NamespaceSDict::Iterator nli(*Doxygen::namespaceSDict);
-  NamespaceDef *nd;
+  const NamespaceDef *nd;
   for (nli.toFirst();(nd=nli.current());++nli)
     generatePerlModForNamespace(nd);
   m_output.closeList();
@@ -2224,7 +2217,7 @@ bool PerlModGenerator::generatePerlModOutput()
   for (;(fn=fnli.current());++fnli)
   {
     FileNameIterator fni(*fn);
-    FileDef *fd;
+    const FileDef *fd;
     for (;(fd=fni.current());++fni)
       generatePerlModForFile(fd);
   }
@@ -2232,7 +2225,7 @@ bool PerlModGenerator::generatePerlModOutput()
 
   m_output.openList("groups");
   GroupSDict::Iterator gli(*Doxygen::groupSDict);
-  GroupDef *gd;
+  const GroupDef *gd;
   for (;(gd=gli.current());++gli)
   {
     generatePerlModForGroup(gd);
@@ -2282,13 +2275,13 @@ bool PerlModGenerator::createOutputDir(QDir &perlModDir)
       dir.setPath(QDir::currentDirPath());
       if (!dir.mkdir(outputDirectory))
       {
-	err("tag OUTPUT_DIRECTORY: Output directory `%s' does not "
+	err("tag OUTPUT_DIRECTORY: Output directory '%s' does not "
 	    "exist and cannot be created\n",outputDirectory.data());
 	exit(1);
       }
       else
       {
-	msg("Notice: Output directory `%s' does not exist. "
+	msg("Notice: Output directory '%s' does not exist. "
 	    "I have created it for you.\n", outputDirectory.data());
       }
       dir.cd(outputDirectory);
