@@ -1,12 +1,12 @@
 /******************************************************************************
  *
- * 
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ *
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -18,56 +18,61 @@
 #ifndef TRANSLATOR_H
 #define TRANSLATOR_H
 
-#include "qtbc.h"
 #include "classdef.h"
-#include "util.h"
 #include "config.h"
+#include "vhdldocgen.h"
 
 /** Abstract base class for all translatable text fragments. */
 class Translator
 {
-  private:
-    /* Tables for encoding conversions. */
-    static const char Win1250ToISO88592Tab[];
-    static const char ISO88592ToWin1250Tab[];
-    static const unsigned char Koi8RToWindows1251Tab[128];
-    static const unsigned char Windows1251ToKoi8RTab[128];
-
-  protected:
-    /*! Returns the string converted from windows-1250 to iso-8859-2. */
-    QCString Win1250ToISO88592(const QCString & sInput);
-    
-    /*! Returns the string converted from iso-8859-2 to windows-1250. */
-    QCString ISO88592ToWin1250(const QCString & sInput);
-    
-    /*! Returns the string converted from koi8-r to windows-1251. */
-    QCString Koi8RToWindows1251( const QCString & sInput );
-
-    /*! Returns the string converted from windows-1251 to koi8-r. */
-    QCString Windows1251ToKoi8R( const QCString & sInput );
-    
-    /*! Returns the string converted from Japanese-EUC to SJIS. */
-    QCString JapaneseEucToSjis( const QCString & sInput );
-
   public:
 
     /*! This method is used to provide warning message that is displayed
-     *  if the user chooses a language whose translation is not up to date. 
+     *  if the user chooses a language whose translation is not up to date.
      *  It is implemented by the adapter classes.
      */
     virtual QCString updateNeededMessage() { return ""; }
     virtual ~Translator() {}
-   
+
     // Please, have a look at comments inside the translator_en.h file
     // to learn the meaning of the following methods.  The translator_en.h
-    // file contains the TranslatorEnglish implementation, which is 
+    // file contains the TranslatorEnglish implementation, which is
     // always up-to-date (by definition).
-   
+
     // --- Language control methods -------------------
-    
+
     virtual QCString idLanguage() = 0;
     virtual QCString latexLanguageSupportCommand() = 0;
-    virtual QCString idLanguageCharset() = 0;
+    /*!
+     * Sets the LaTeX font encoding to be used. The default is set to `T1`,
+     * in case another font encoding has to be used this can be specified with
+     * this routine. In case no font encoding is required the empty string
+     * can be returned.
+     */
+    virtual QCString latexFontenc() { return "T1"; }
+    /*!
+     * Sets the commands to be insered directly after the `\\begin{document}`
+     * in the LaTeX document.
+     */
+    virtual QCString latexDocumentPre() { return ""; }
+    /*!
+     * Sets the commands to be insered directly before the `\\end{document}`
+     * in the LaTeX document.
+     */
+    virtual QCString latexDocumentPost() { return ""; }
+    /*!
+     * Set the name to be used as latex command.
+     */
+    virtual QCString latexCommandName()
+    {
+      QCString latex_command = Config_getString(LATEX_CMD_NAME);
+      if (latex_command.isEmpty()) latex_command = "latex";
+      if (Config_getBool(USE_PDFLATEX))
+      {
+        if (latex_command == "latex") latex_command = "pdflatex";
+      }
+      return latex_command;
+    }
 
     // --- Language translation methods -------------------
 
@@ -110,14 +115,14 @@ class Translator
     virtual QCString trRelatedPagesDescription() = 0;
     virtual QCString trModulesDescription() = 0;
     //virtual QCString trNoDescriptionAvailable() = 0;
-    
-    // index titles (the project name is prepended for these) 
+
+    // index titles (the project name is prepended for these)
 
     virtual QCString trDocumentation() = 0;
     virtual QCString trModuleIndex() = 0;
     virtual QCString trHierarchicalIndex() = 0;
     virtual QCString trCompoundIndex() = 0;
-    virtual QCString trFileIndex() = 0; 
+    virtual QCString trFileIndex() = 0;
     virtual QCString trModuleDocumentation() = 0;
     virtual QCString trClassDocumentation() = 0;
     virtual QCString trFileDocumentation() = 0;
@@ -125,21 +130,21 @@ class Translator
     virtual QCString trPageDocumentation() = 0;
     virtual QCString trReferenceManual() = 0;
     virtual QCString trDefines() = 0;
-    virtual QCString trFuncProtos() = 0;
+    //virtual QCString trFuncProtos() = 0;
     virtual QCString trTypedefs() = 0;
     virtual QCString trEnumerations() = 0;
     virtual QCString trFunctions() = 0;
     virtual QCString trVariables() = 0;
     virtual QCString trEnumerationValues() = 0;
     virtual QCString trDefineDocumentation() = 0;
-    virtual QCString trFunctionPrototypeDocumentation() = 0;
+    //virtual QCString trFunctionPrototypeDocumentation() = 0;
     virtual QCString trTypedefDocumentation() = 0;
     virtual QCString trEnumerationTypeDocumentation() = 0;
     virtual QCString trFunctionDocumentation() = 0;
     virtual QCString trVariableDocumentation() = 0;
     virtual QCString trCompounds() = 0;
     virtual QCString trGeneratedAt(const char *date,const char *projName) = 0;
-    virtual QCString trWrittenBy() = 0;
+    //virtual QCString trWrittenBy() = 0;
     virtual QCString trClassDiagram(const char *clName) = 0;
     virtual QCString trForInternalUseOnly() = 0;
     //virtual QCString trReimplementedForInternalReasons() = 0;
@@ -156,17 +161,17 @@ class Translator
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990307
 //////////////////////////////////////////////////////////////////////////
-    
+
     virtual QCString trNamespaceList() = 0;
     virtual QCString trNamespaceListDescription(bool extractAll) = 0;
     virtual QCString trFriends() = 0;
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990405
 //////////////////////////////////////////////////////////////////////////
-    
+
     virtual QCString trRelatedFunctionDocumentation() = 0;
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990425
 //////////////////////////////////////////////////////////////////////////
@@ -177,7 +182,7 @@ class Translator
 
     virtual QCString trFileReference(const char *fileName) = 0;
     virtual QCString trNamespaceReference(const char *namespaceName) = 0;
-    
+
     virtual QCString trPublicMembers() = 0;
     virtual QCString trPublicSlots() = 0;
     virtual QCString trSignals() = 0;
@@ -210,7 +215,7 @@ class Translator
 
     virtual QCString trGeneratedFromFiles(ClassDef::CompoundType compType,
                                           bool single) = 0;
-    virtual QCString trAlphabeticalList() = 0;
+    //virtual QCString trAlphabeticalList() = 0;
 
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990901
@@ -257,7 +262,7 @@ class Translator
 //////////////////////////////////////////////////////////////////////////
 // new since 1.1.0
 //////////////////////////////////////////////////////////////////////////
-    
+
     virtual QCString trNote() = 0;
     virtual QCString trPublicTypes() = 0;
     virtual QCString trPublicAttribs() = 0;
@@ -285,7 +290,7 @@ class Translator
     virtual QCString trAttention() = 0;
     virtual QCString trInclByDepGraph() = 0;
     virtual QCString trSince() = 0;
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.1.5
 //////////////////////////////////////////////////////////////////////////
@@ -293,11 +298,11 @@ class Translator
     virtual QCString trLegendTitle() = 0;
     virtual QCString trLegendDocs() = 0;
     virtual QCString trLegend() = 0;
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.2.0
 //////////////////////////////////////////////////////////////////////////
-    
+
     virtual QCString trTest() = 0;
     virtual QCString trTestList() = 0;
 
@@ -305,7 +310,7 @@ class Translator
 // new since 1.2.1
 //////////////////////////////////////////////////////////////////////////
 
-    virtual QCString trDCOPMethods() = 0;
+    //virtual QCString trDCOPMethods() = 0;
 
 //////////////////////////////////////////////////////////////////////////
 // new since 1.2.2
@@ -326,11 +331,11 @@ class Translator
     virtual QCString trPackages() = 0;
     //virtual QCString trPackageDocumentation() = 0;
     virtual QCString trDefineValue() = 0;
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.2.5
 //////////////////////////////////////////////////////////////////////////
-    
+
     virtual QCString trBug() = 0;
     virtual QCString trBugList() = 0;
 
@@ -338,10 +343,65 @@ class Translator
 // new since 1.2.6
 //////////////////////////////////////////////////////////////////////////
 
+
+    /*! Used as ansicpg for RTF file
+     *
+     * The following table shows the correlation of Charset name, Charset Value and
+     * <pre>
+     * Codepage number:
+     * Charset Name       Charset Value(hex)  Codepage number
+     * ------------------------------------------------------
+     * DEFAULT_CHARSET           1 (x01)
+     * SYMBOL_CHARSET            2 (x02)
+     * OEM_CHARSET             255 (xFF)
+     * ANSI_CHARSET              0 (x00)            1252
+     * RUSSIAN_CHARSET         204 (xCC)            1251
+     * EE_CHARSET              238 (xEE)            1250
+     * GREEK_CHARSET           161 (xA1)            1253
+     * TURKISH_CHARSET         162 (xA2)            1254
+     * BALTIC_CHARSET          186 (xBA)            1257
+     * HEBREW_CHARSET          177 (xB1)            1255
+     * ARABIC _CHARSET         178 (xB2)            1256
+     * SHIFTJIS_CHARSET        128 (x80)             932
+     * HANGEUL_CHARSET         129 (x81)             949
+     * GB2313_CHARSET          134 (x86)             936
+     * CHINESEBIG5_CHARSET     136 (x88)             950
+     * </pre>
+     *
+     */
     virtual QCString trRTFansicp() = 0;
+
+    /*! Character sets
+     *  <pre>
+     *   0 — ANSI
+     *   1 — Default
+     *   2 — Symbol
+     *   3 — Invalid
+     *  77 — Mac
+     * 128 — Shift Jis
+     * 129 — Hangul
+     * 130 — Johab
+     * 134 — GB2312
+     * 136 — Big5
+     * 161 — Greek
+     * 162 — Turkish
+     * 163 — Vietnamese
+     * 177 — Hebrew
+     * 178 — Arabic
+     * 179 — Arabic Traditional
+     * 180 — Arabic user
+     * 181 — Hebrew user
+     * 186 — Baltic
+     * 204 — Russian
+     * 222 — Thai
+     * 238 — Eastern European
+     * 254 — PC 437
+     * 255 — OEM
+     * </pre>
+     */
     virtual QCString trRTFCharSet() = 0;
     virtual QCString trRTFGeneralIndex() = 0;
-   
+
     // Translation of the word
 
     virtual QCString trClass(bool first_capital, bool singular) = 0;
@@ -356,7 +416,7 @@ class Translator
 //////////////////////////////////////////////////////////////////////////
 // new since 1.2.7
 //////////////////////////////////////////////////////////////////////////
-        
+
     virtual QCString trAuthor(bool first_capital, bool singular) = 0;
 
 //////////////////////////////////////////////////////////////////////////
@@ -371,7 +431,7 @@ class Translator
 
     virtual QCString trImplementedFromList(int numEntries) = 0;
     virtual QCString trImplementedInList(int numEntries) = 0;
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.2.16
 //////////////////////////////////////////////////////////////////////////
@@ -383,7 +443,7 @@ class Translator
 //////////////////////////////////////////////////////////////////////////
 
     virtual QCString trDeprecatedList() = 0;
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.2.18
 //////////////////////////////////////////////////////////////////////////
@@ -400,7 +460,7 @@ class Translator
     virtual QCString trStaticPackageMembers() = 0;
     virtual QCString trPackageAttribs() = 0;
     virtual QCString trStaticPackageAttribs() = 0;
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.3.1
 //////////////////////////////////////////////////////////////////////////
@@ -412,7 +472,7 @@ class Translator
 // new since 1.3.3
 //////////////////////////////////////////////////////////////////////////
 
-    virtual QCString trSearchForIndex() = 0;
+    //virtual QCString trSearchForIndex() = 0;
     virtual QCString trSearchResultsTitle() = 0;
     virtual QCString trSearchResults(int numDocuments) = 0;
     virtual QCString trSearchMatches() = 0;
@@ -443,14 +503,14 @@ class Translator
 //////////////////////////////////////////////////////////////////////////
 // new since 1.4.6
 //////////////////////////////////////////////////////////////////////////
-    
+
     virtual QCString trCallerGraph() = 0;
     virtual QCString trEnumerationValueDocumentation() = 0;
 
 //////////////////////////////////////////////////////////////////////////
 // new since 1.5.4
 //////////////////////////////////////////////////////////////////////////
-    
+
     virtual QCString trMemberFunctionDocumentationFortran() = 0;
     virtual QCString trCompoundListFortran() = 0;
     virtual QCString trCompoundMembersFortran() = 0;
@@ -491,13 +551,13 @@ class Translator
 // new since 1.6.3
 //////////////////////////////////////////////////////////////////////////
 
-    virtual QCString trDirDependency(const char *name) = 0;
+    //virtual QCString trDirDependency(const char *name) = 0;
     virtual QCString trFileIn(const char *name) = 0;
     virtual QCString trIncludesFileIn(const char *name) = 0;
     virtual QCString trDateTime(int year,int month,int day,int dayOfWeek,
                                 int hour,int minutes,int seconds,
                                 bool includeTime) = 0;
-                       
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.7.5
 //////////////////////////////////////////////////////////////////////////
@@ -530,6 +590,62 @@ class Translator
     virtual QCString trMethodDocumentation() = 0;
     virtual QCString trDesignOverview() = 0;
 
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.4
+//////////////////////////////////////////////////////////////////////////
+
+    virtual QCString trInterfaces() = 0;
+    virtual QCString trServices() = 0;
+    virtual QCString trConstantGroups() = 0;
+    virtual QCString trConstantGroupReference(const char *namespaceName) = 0;
+    virtual QCString trServiceReference(const char *sName) = 0;
+    virtual QCString trSingletonReference(const char *sName) = 0;
+    virtual QCString trServiceGeneratedFromFiles(bool single) = 0;
+    virtual QCString trSingletonGeneratedFromFiles(bool single) = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.15
+//////////////////////////////////////////////////////////////////////////
+
+    virtual QCString trDesignUnitHierarchy() = 0;
+    virtual QCString trDesignUnitList() = 0;
+    virtual QCString trDesignUnitMembers() = 0;
+    virtual QCString trDesignUnitListDescription() = 0;
+    virtual QCString trDesignUnitIndex() = 0;
+    virtual QCString trDesignUnits() = 0;
+    virtual QCString trFunctionAndProc() = 0;
+    virtual QCString trVhdlType(uint64 type,bool single) = 0;
+    virtual QCString trCustomReference(const char *name) = 0;
+
+    virtual QCString trConstants() = 0;
+    virtual QCString trConstantDocumentation() = 0;
+    virtual QCString trSequences() = 0;
+    virtual QCString trSequenceDocumentation() = 0;
+    virtual QCString trDictionaries() = 0;
+    virtual QCString trDictionaryDocumentation() = 0;
+    virtual QCString trSliceInterfaces() = 0;
+    virtual QCString trInterfaceIndex() = 0;
+    virtual QCString trInterfaceList() = 0;
+    virtual QCString trInterfaceListDescription() = 0;
+    virtual QCString trInterfaceHierarchy() = 0;
+    virtual QCString trInterfaceHierarchyDescription() = 0;
+    virtual QCString trInterfaceDocumentation() = 0;
+    virtual QCString trStructs() = 0;
+    virtual QCString trStructIndex() = 0;
+    virtual QCString trStructList() = 0;
+    virtual QCString trStructListDescription() = 0;
+    virtual QCString trStructDocumentation() = 0;
+    virtual QCString trExceptionIndex() = 0;
+    virtual QCString trExceptionList() = 0;
+    virtual QCString trExceptionListDescription() = 0;
+    virtual QCString trExceptionHierarchy() = 0;
+    virtual QCString trExceptionHierarchyDescription() = 0;
+    virtual QCString trExceptionDocumentation() = 0;
+    virtual QCString trCompoundReferenceSlice(const char *clName, ClassDef::CompoundType compType, bool isLocal) = 0;
+    virtual QCString trOperations() = 0;
+    virtual QCString trOperationDocumentation() = 0;
+    virtual QCString trDataMembers() = 0;
+    virtual QCString trDataMemberDocumentation() = 0;
 };
 
 #endif
